//Name: Menger Sponge
//File: asMengerSponge.ls
//Version: 1.0
//Last Update 03/18/07
//Author: Artur Wawrzonkiewicz
//URL and contact: www.artssphere.com
//License: FREE
//
//This program is free to distribute and modify as long as
//the name of the original author (Artur Wawrzonkiewicz) 
//is mentioned. 

@warnings
@name Menger Sponge
@script modeler

max_level = 3;

main
{
        ct = <0,0,0>;     //center
        lv = 0;           //current level

        merge = true;

        reqbegin("Menger Sponge");
                reqsize(200,149);

                c1 = ctlinteger("Level",max_level);
                ctlposition(c1,40,10);
                
                c2 = ctlnumber("Size",1);
                ctlposition(c2,46,38);

                c3 = ctlcheckbox("Merge points & polygons",merge);
                ctlposition(c3,50,65);                

                return if !reqpost();

                max_level = getvalue(c1);
                
                if(max_level < 1) max_level = 1;
                
                u = getvalue(c2) / 2;           //unit
                merge = getvalue(c3);
        reqend();

        moninit(22);

        editbegin();
        
        subdivide(ct,u,1);
        
        editend();
       
        if(merge)
        {
                mergepoints();  //merge points
                monstep();
                unifypols();    //unify overlapping polygons (does not remove polygons hidden inside of the structure)
                monstep();
        }
        monend();        
}

subdivide: ct,u,lv
{
        o = u * 2 / 3;          //center offset
        u = u / 3;              //new unit
        tc = <0,0,0>;           //new center (temporary center)

        if(lv==2) monstep();

        //precompute common expressions
        cxp = ct.x + o;
        cxm = ct.x - o;

        cyp = ct.y + o;
        cym = ct.y - o;

        czp = ct.z + o;
        czm = ct.z - o;

        if(lv == max_level)
        {
                //TOP
                tc.y = cyp;
                
                tc.x = cxm;
                tc.z = ct.z;
                box(tc,u);

                tc.x = cxp;
                box(tc,u);

                tc.x = ct.x;
                tc.z = czm;
                box(tc,u);

                tc.z = czp;
                box(tc,u);

                tc.x = cxm;
                tc.z = czm;
                box(tc,u);

                tc.x = cxp;
                box(tc,u);
                
                tc.x = cxm;
                tc.z = czp;
                box(tc,u);

                tc.x = cxp;
                box(tc,u);
                
                //BOTTOM
                tc.y = cym;

                tc.x = cxm;
                tc.z = ct.z;
                box(tc,u);
                
                tc.x = cxp;
                box(tc,u);
                
                tc.x = ct.x;
                tc.z = czm;
                box(tc,u);
                
                tc.z = czp;
                box(tc,u);
                
                tc.x = cxm;
                tc.z = czm;
                box(tc,u);
                
                tc.x = cxp;
                box(tc,u);
                
                tc.x = cxm;
                tc.z = czp;
                box(tc,u);
                
                tc.x = cxp;
                box(tc,u);
                
                //MIDDLE
                tc.y = ct.y;

                tc.x = cxm;
                tc.z = czm;
                box(tc,u);
                
                tc.x = cxp;
                box(tc,u);
                
                tc.x = cxm;
                tc.z = czp;
                box(tc,u);
                
                tc.x = cxp;
                box(tc,u);                
        }
        else
        {
                //TOP
                tc.y = cyp;

                tc.x = cxm;
                tc.z = ct.z;
                subdivide(tc,u,lv+1);

                tc.x = cxp;
                subdivide(tc,u,lv+1);

                tc.x = ct.x;
                tc.z = czm;
                subdivide(tc,u,lv+1);

                tc.z = czp;
                subdivide(tc,u,lv+1);

                tc.x = cxm;
                tc.z = czm;
                subdivide(tc,u,lv+1);

                tc.x = cxp;
                subdivide(tc,u,lv+1);

                tc.x = cxm;
                tc.z = czp;
                subdivide(tc,u,lv+1);

                tc.x = cxp;
                subdivide(tc,u,lv+1);

                //BOTTOM
                tc.y = cym;

                tc.x = cxm;
                tc.z = ct.z;
                subdivide(tc,u,lv+1);

                tc.x = cxp;
                subdivide(tc,u,lv+1);

                tc.x = ct.x;
                tc.z = czm;
                subdivide(tc,u,lv+1);

                tc.z = czp;
                subdivide(tc,u,lv+1);

                tc.x = cxm;
                tc.z = czm;
                subdivide(tc,u,lv+1);

                tc.x = cxp;
                subdivide(tc,u,lv+1);

                tc.x = cxm;
                tc.z = czp;
                subdivide(tc,u,lv+1);

                tc.x = cxp;
                subdivide(tc,u,lv+1);

                //MIDDLE
                tc.y = ct.y;

                tc.x = cxm;
                tc.z = czm;
                subdivide(tc,u,lv+1);

                tc.x = cxp;
                subdivide(tc,u,lv+1);

                tc.x = cxm;
                tc.z = czp;
                subdivide(tc,u,lv+1);

                tc.x = cxp;
                subdivide(tc,u,lv+1);
        }
}

box: c,o
{
                //precompute common expressions
                cxp = c.x + o;
                cxm = c.x - o;

                cyp = c.y + o;
                cym = c.y - o;

                czp = c.z + o;
                czm = c.z - o;

                p1 = addpoint(<cxm,cyp,czp>);
                p2 = addpoint(<cxp,cyp,czp>);
                p3 = addpoint(<cxp,cyp,czm>);                
                p4 = addpoint(<cxm,cyp,czm>);
                
                p5 = addpoint(<cxm,cym,czp>);
                p6 = addpoint(<cxp,cym,czp>);
                p7 = addpoint(<cxp,cym,czm>);                
                p8 = addpoint(<cxm,cym,czm>);
                
                p[1] = p1;
                p[2] = p2;
                p[3] = p3;
                p[4] = p4;
                addpolygon(p);

                p[1] = p8;
                p[2] = p7;
                p[3] = p6;
                p[4] = p5;
                addpolygon(p);

                p[1] = p4;
                p[2] = p3;
                p[3] = p7;
                p[4] = p8;
                addpolygon(p);

                p[1] = p3;
                p[2] = p2;
                p[3] = p6;
                p[4] = p7;
                addpolygon(p);

                p[1] = p2;
                p[2] = p1;
                p[3] = p5;
                p[4] = p6;
                addpolygon(p);

                p[1] = p1;
                p[2] = p4;
                p[3] = p8;
                p[4] = p5;
                addpolygon(p);

}